package com.inteligr8.junit5;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.junit.jupiter.api.Assertions;

public class AssertRegex {
	
	public static void assertMatches(String expectedRegex, String actual) {
		assertMatches(expectedRegex, actual, null);
	}
	
	public static void assertMatches(String expectedRegex, String actual, String message) {
		Pattern pattern = Pattern.compile(expectedRegex);
		assertMatches(pattern, actual, message);
	}
	
	public static void assertMatches(Pattern expected, String actual) {
		assertMatches(expected, actual, null);
	}
	
	public static void assertMatches(Pattern expected, String actual, String message) {
		assertMatches(false, expected, actual, message);
	}
	
	public static void assertNotMatches(String expectedRegex, String actual) {
		assertNotMatches(expectedRegex, actual, null);
	}
	
	public static void assertNotMatches(String expectedRegex, String actual, String message) {
		Pattern pattern = Pattern.compile(expectedRegex);
		assertNotMatches(pattern, actual, message);
	}
	
	public static void assertNotMatches(Pattern expected, String actual) {
		assertNotMatches(expected, actual, null);
	}
	
	public static void assertNotMatches(Pattern expected, String actual, String message) {
		assertMatches(true, expected, actual, message);
	}
	
	private static void assertMatches(boolean negate, Pattern expected, String actual, String message) {
		Matcher matcher = expected.matcher(actual);
		if (matcher.matches() == negate) {
			message = message == null ? "" : (message + "; ");
			message += "expression of expected string: <" + expected.toString() + "> but actual string: <" + actual + ">";
			Assertions.fail(message);
		}
	}
	
	
	
	public static void assertFind(String expectedRegex, int expectedCount, String actual) {
		assertFind(expectedRegex, expectedCount, actual, null);
	}
	
	public static void assertFind(String expectedRegex, int expectedCount, String actual, String message) {
		Pattern pattern = Pattern.compile(expectedRegex);
		assertFind(pattern, expectedCount, actual, message);
	}
	
	public static void assertFind(Pattern expected, int expectedCount, String actual) {
		assertFind(expected, expectedCount, actual, null);
	}
	
	public static void assertFind(Pattern expected, int expectedCount, String actual, String message) {
		Matcher matcher = expected.matcher(actual);
		int count = 0;
		for (count = 0; matcher.find(); count++)
			;
		if (expectedCount < 0) Assertions.assertTrue(-expectedCount <= count, message);
		else Assertions.assertEquals(expectedCount, count, message);
	}

}
